/*

	CGUIIrrObjectTreeViewer by Jonas Petersen a.k.a. "jox"
	http://development.mindfloaters.de/

	An addon class for the Irrlicht Engine by Nikolaus Gebhardt
	http://irrlicht.sourceforge.net/

	Version 0.1 - 19th March 2005 - Irrlicht 0.8

	A GUI class that displays the hierarchy of all scene nodes and GUI elements present in the engine.


	Usage:
	------

	To use it add the files CGUIIrrObjectTreeViewer.h and CGUIIrrObjectTreeViewer.cpp to your project
	and add some code similar to the following:

	---------------------------------------------------------------------

	#include "CGUIIrrObjectTreeViewer.h"

	using namespace irr;

	// ...

	gui::CGUIIrrObjectTreeViewer* objViewer =
		new gui::CGUIIrrObjectTreeViewer(
			guienv,                             // GUIEnvironment
			guienv->getRootGUIElement(),        // parent
			-1,                                 // id
			core::rect<s32>(150,100,490,400),   // position & size
			smgr,                               // SceneManager
			true                                // create a window containing the viewer (default).
			                                    // if set to false it can be integrated in
			                                    // another gui element
		);

	// drop directly because it get grabed by the parent (root gui element in this case).
	objViewer->drop();

	---------------------------------------------------------------------

	There are two options (last parameter "bool createWindow"):
	1) true (default): Create it as a GUIWindow. A movable window containing the viewer will be created.
	2) false: Create stand-alone. Only the listbox with the buttons will be created relative to the
	   given parent.

	to set the width of the indentation in spaces use e.g. (default is 8):

	objViewer->setIndentWidth(4);

	Please note that the "DebugNames" of objects are displayed. These are only available in debug build
	of the engine. However, unfortunately not all Irrlicht objects have a "DebugName" currently
	(Irrlicht 0.8). If an object has none, then "NO_DEBUG_NAME" will be displayed instead. This is also
	the case for ALL objects in Release build of the engine (no _DEBUG macro).

	Debug names are set in the constructor of classes that derive from irr::IUnknown like this:

		#ifdef _DEBUG
		setDebugName("CExampleClass");
		#endif


	Currently it is only tested under Windows with MSVC .NET.


	Version History:
	----------------

	Version 0.1 - 19th March 2005
	- Initial release.

*/

#include "CGUIIrrObjectTreeViewer.h"


namespace irr {
namespace gui {


CGUIIrrObjectTreeViewer::CGUIIrrObjectTreeViewer(IGUIEnvironment* environment, IGUIElement* parent, s32 id,
	core::rect<s32> rect, scene::ISceneManager* smgr, bool createWindow)
 : IGUIElement(environment, parent, id, rect), SceneManager(smgr), IndentWidth(8), Window(NULL)
{
	#ifdef _DEBUG
	setDebugName("CGUIIrrObjectTreeViewer");
	#endif

	if (createWindow) {
		Window = Environment->addWindow(RelativeRect, false, L"IrrObject Viewer by jox", Environment->getRootGUIElement());

		grab();
		remove();
		Window->addChild(this);
		Parent = Window;
		drop();
		
		s32 titel_bar_height = Environment->getSkin()->getSize(EGDS_WINDOW_BUTTON_WIDTH) + 4;
		RelativeRect = core::rect<s32>(3, 3 + titel_bar_height, RelativeRect.getWidth()-3, RelativeRect.getHeight()-3);
		updateAbsolutePosition();
	}

	s32 button_height = 20;
	core::rect<s32> r(0, 0, RelativeRect.getWidth(), RelativeRect.getHeight()-button_height);

	ListBox = Environment->addListBox(r, this, -1, true);
	ListBox->setIconFont(Environment->getSkin()->getFont());

	r.UpperLeftCorner.Y = r.LowerRightCorner.Y;
	r.LowerRightCorner.Y += button_height;
	r.LowerRightCorner.X = RelativeRect.getWidth() / 2;
	Environment->addButton(r, this, 101, L"SceneNodes");

	r.UpperLeftCorner.X = r.LowerRightCorner.X;
	r.LowerRightCorner.X = RelativeRect.getWidth();

	Environment->addButton(r, this, 102, L"GUIElements");

	for (s32 i=0; i<IndentWidth; i++) {
		IndentString += L" ";
	}

	// display scene nodes as default
	updateSceneNodes();
}

CGUIIrrObjectTreeViewer::~CGUIIrrObjectTreeViewer(void)
{
}

IGUIWindow* CGUIIrrObjectTreeViewer::getWindow() {
	return Window;
}

bool CGUIIrrObjectTreeViewer::OnEvent(SEvent event) {
	if (event.EventType == EET_GUI_EVENT && event.GUIEvent.EventType == EGET_BUTTON_CLICKED) {
		if (event.GUIEvent.Caller->getID() == 101)
			updateSceneNodes();
		else
			updateGUIElements();
	}
	return true;
}

void CGUIIrrObjectTreeViewer::setIndentWidth(s32 width) {
	IndentWidth = width;
}

void CGUIIrrObjectTreeViewer::updateGUIElements() {
	ListBox->clear();
	ObjCount = 0;

	ListBox->addItem(L"GUIElement tree:");
	ListBox->addItem(L"");

	parseGUIElements(Environment->getRootGUIElement());

	if (ObjCount)
		ListBox->addItem(L"");
	core::stringw text = L"Total elements: ";
	text += ObjCount;
	ListBox->addItem(text.c_str());
}

void CGUIIrrObjectTreeViewer::parseGUIElements(gui::IGUIElement* parent, s32 level) {
	const core::list<gui::IGUIElement*>& list = parent->getChildren();
	core::list<gui::IGUIElement*>::Iterator it = list.begin();
	for (; it!=list.end(); it++) {
		gui::IGUIElement* element = (*it);

		core::stringw text;

		for (s32 i=0; i<level; i++) {
			text += IndentString;
		}

		const c8* debug_name = 0;

		#ifdef _DEBUG
		debug_name = element->getDebugName();
		#endif

		if (debug_name && *debug_name)
			text += debug_name;
		else
			text += L"NO_DEBUG_NAME";

		text += L" (id: ";
		text += element->getID();
		if (*element->getText() != 0) {
			text += L", Text: \"";
			text += element->getText();
			text += L"\"";
		}
		text += L")";

		if (element->isVisible())
			ListBox->addItem(text.c_str());
		else
			ListBox->addItem(text.c_str(), L"*");

		ObjCount++;

		parseGUIElements(element, level+1);
	}
}


void CGUIIrrObjectTreeViewer::updateSceneNodes() {
	ListBox->clear();
	ObjCount = 0;

	ListBox->addItem(L"SceneNode tree:");
	ListBox->addItem(L"");

	parseSceneNodes(SceneManager->getRootSceneNode());

	if (ObjCount)
		ListBox->addItem(L"");
	core::stringw text = L"Total nodes: ";
	text += ObjCount;
	ListBox->addItem(text.c_str());
}

void CGUIIrrObjectTreeViewer::parseSceneNodes(scene::ISceneNode* parent, s32 level) {
	const core::list<scene::ISceneNode*>& list = parent->getChildren();
	core::list<scene::ISceneNode*>::Iterator it = list.begin();
	for (; it!=list.end(); it++) {
		scene::ISceneNode* node = (*it);

		core::stringw text;

		for (s32 i=0; i<level; i++) {
			text += IndentString;
		}

		const c8* debug_name = 0;

		#ifdef _DEBUG
		debug_name = node->getDebugName();
		#endif

		if (debug_name && *debug_name)
			text += debug_name;
		else
			text += L"NO_DEBUG_NAME";

		text += L" (id: ";
		text += node->getID();
		if (*node->getName() != 0) {
			text += L", Name: \"";
			text += node->getName();
			text += L"\"";
		}
		text += L")";

		if (node->isVisible())
			ListBox->addItem(text.c_str());
		else
			ListBox->addItem(text.c_str(), L"*");

		ObjCount++;

		parseSceneNodes(node, level+1);
	}
}

} // namespace irr
} // namespace gui 
